from pathlib import Path

from ..document import Document
from .extractor_cloze import ClozePromptExtractor


def test_cloze_prompt_extractor(tmpdir: Path):
    doc = Document(
        content=r"""

What is the meaning of life? {42}


This is another block without any cloze prompts.

#anki/tag/test_tag

""",
        source_path=tmpdir / "test.md",
    )

    extractor = ClozePromptExtractor().extract_prompts(doc)

    assert len(extractor) == 1
    assert extractor[0].text == r"What is the meaning of life? {{c734::42}}"
    assert extractor[0].tags == ["anki/tag/test_tag"]


import pytest


@pytest.mark.parametrize(
    ("content", "skipped"),
    [
        (
            """```html
         {Code block}
         ```""",
            True,
        ),
        ("""<!-- {HTML comment} -->""", True),
        ("""{Cloze}""", False),
        (
            """```html
Some content

Content in another {block}
        ```""",
            True,
        ),
    ],
)
def test_ignore_block_types(content: str, skipped: bool):
    doc = Document(content=content, source_path=Path("test.md"))
    extractor = ClozePromptExtractor().extract_prompts(doc)

    if skipped:
        assert len(extractor) == 0
    else:
        assert len(extractor) == 1



from collections.abc import Sequence
from dataclasses import dataclass

import genanki

from memium.utils.extract_terms import get_terms_surrounded_by_underscores

from ...utils.hash_cleaned_str import hash_str_to_int
from .anki_prompt import AnkiPrompt


@dataclass(frozen=True)
class AnkiQA(AnkiPrompt):
    base_deck: str
    tags: Sequence[str]
    question: str
    answer: str
    css: str
    uuid: int  # UUID used for scheduling. If a new note is added with the same uuid, it will override the old note.

    @property
    def genanki_model(self) -> genanki.Model:
        model_fields = [
            {"name": "Question"},
            {"name": "Answer"},
            {"name": "Extra"},
            {"name": "UUID"},
        ]

        QUESTION_STR = r"{{ Question }}"
        TTS_QUESTION_STR = r"{{ tts en_US voices=Apple_Samantha speed=1.05:Question }}"

        ANSWER_STR = r"{{ Answer }}"
        TTS_ANSWER_STR = r"{{ tts en_US voices=Apple_Samantha speed=1.05:Answer }}"

        EXTRA_STR = r"{{ Extra }}"

        model_template = [
            {
                "name": "Ankdown QA Card with UUID",
                "qfmt": f"""
<div class="extra">
    {EXTRA_STR}
</div>
<div class="front">
    {QUESTION_STR}{TTS_QUESTION_STR}
</div>
            """,
                "afmt": f"""
<div class="back">
    <div class="extra">
        {EXTRA_STR}
    </div>
    <div class="question">
        {QUESTION_STR}
    </div>
    <div class="answer">
        {ANSWER_STR}{TTS_ANSWER_STR}
    </div>
</div>
            """,
            }
        ]

        return genanki.Model(
            model_id=hash_str_to_int("Ankdown QA with UUID"),
            name=("Ankdown QA with UUID"),
            fields=model_fields,
            templates=model_template,
            css=self.css,
            model_type=0,
        )

    @property
    def _extra_field_content(self) -> str:
        return "" if not self.edit_url else edit_button(self.edit_url)

    def to_genanki_note(self) -> genanki.Note:
        return genanki.Note(
            guid=str(self.uuid),
            model=self.genanki_model,
            fields=[
                self._to_html(self.question),
                self._to_html(self.answer),
                self._extra_field_content,
                str(self.uuid),
            ],
            tags=self.tags,
        )

    @property
    def deck(self) -> str:
        base_deck = super().deck
        wiki_links = get_terms_surrounded_by_underscores(self.question)
        wiki_subdeck = "-".join(sorted(wiki_links))

        if wiki_subdeck != "":
            return f"{base_deck}::{wiki_subdeck}"
        return base_deck


def edit_button(url: str) -> str:
    return f"""<a href="{url}" style="background-color: #5f0069;
border: none;
color: white;
padding: 0.8em;
text-align: center;
text-decoration: none;
font-size: 0.8em;
font-family: 'Inter', sans-serif;
float: right;
border-radius: 8px;
opacity: 0.8;
">Obsidian</a>"""



from dataclasses import dataclass

from memium.source.document import Document
from memium.source.prompts.prompt_qa import QAFromDoc, QAPrompt


@dataclass(frozen=True)
class FakeQAPrompt(QAPrompt):
    @property
    def edit_url(self) -> str:
        return ""

    def to_qa_from_doc(self, doc: Document, line_nr: int) -> QAPrompt:
        return QAFromDoc(
            parent_doc=doc, line_nr=line_nr, question=self.question, answer=self.answer
        )

    def __repr__(self) -> str:
        return f"{self.question!r}\n{self.answer!r})"



import logging
import sys
import time
from datetime import datetime
from functools import partial
from pathlib import Path
from typing import Annotated, Optional

import typer

from memium.core import main

log = logging.getLogger(__name__)

app = typer.Typer()


@app.command()
def cli(
    input_dir: Annotated[
        Path,
        typer.Option(
            help="Where to extract prompts from.",
            dir_okay=True,
            file_okay=False,
            exists=True,
            readable=True,
            writable=True,
        ),
    ],
    watch_seconds: Annotated[
        Optional[int],  # noqa: UP007
        typer.Option(help="Keep running, updating Anki deck every [ARG] seconds"),
    ] = None,
    deck_name: Annotated[
        str, typer.Option(help="Anki path to deck, e.g. 'Parent deck::Child deck'")
    ] = "Memium",
    max_deletions_per_run: Annotated[
        int,
        typer.Option(
            help="Maximum number of cards to delete per sync to avoid unintentional deletions. If exceeded, raises error."
        ),
    ] = 50,
    push_all: Annotated[
        bool,
        typer.Option(
            help="Push all prompts to Anki, not just the diff. Useful if you have e.g. changed your CSS template. Note that this does not change scheduling, nor delete prompts that no longer exist in your markdown."
        ),
    ] = False,
    dry_run: Annotated[
        bool, typer.Option(help="Don't update via AnkiConnect, just log what would happen")
    ] = False,
    skip_sync: Annotated[
        bool, typer.Option(help="Skip all syncing, useful for smoketesting of the interface")
    ] = False,
):
    start_time = datetime.now()
    config_dir = input_dir / ".memium"
    config_dir.mkdir(exist_ok=True)

    logging.basicConfig(
        handlers=[
            logging.StreamHandler(sys.stdout),
            logging.FileHandler(config_dir / "memium.log"),
        ],
        level=logging.INFO,
        format="%(asctime)s [%(levelname)s] %(name)s: %(message)s",
        datefmt="%Y/&m/%d %H:%M:%S",
    )

    if skip_sync:
        log.info("Skipping sync")
        return

    main_fn = partial(
        main,
        base_deck=deck_name,
        input_dir=input_dir,
        max_deletions_per_run=max_deletions_per_run,
        dry_run=dry_run,
        push_all=push_all,
    )
    main_fn()

    if watch_seconds:
        log.info(
            f"Sync complete in {(datetime.now() - start_time).total_seconds()} seconds, sleeping for {watch_seconds} seconds"
        )
        time.sleep(watch_seconds)
        main_fn()


if __name__ == "__main__":
    app()



import pathlib
from dataclasses import dataclass

import pytest

from .markdown_parser import to_html


@dataclass(frozen=True)
class Ex:
    given: str
    # Required setup

    then: str
    # What the expected result is


@pytest.mark.parametrize(
    ("example"),
    [
        Ex("_Method_s", "<em>Method</em>s")  # Example one
    ],
)
def test_markdown_parser(example: Ex):
    assert to_html(example.given) == f"<p>{example.then}</p>"


def test_code_block_parsing(snapshot: str, tmp_path: pathlib.Path):
    contents = pathlib.Path(__file__).parent / "code_block.md"
    parsed = to_html(contents.read_text())
    path = tmp_path / "test.htm"
    path.write_text(parsed)
    assert parsed == snapshot
